@@ -216,3 +216,19 @@ empathy_notify_manager_notification_is_enabled  (EmpathyNotifyManager *self)
 
   return TRUE;
 }
+
+NotifyNotification *
+empathy_notify_manager_create_notification (const gchar *summary,
+    const char *body,
+    const gchar *icon)
+{
+  NotifyNotification *notification;
+
+  notification = notify_notification_new (summary, body, icon);
+
+  notify_notification_set_hint (notification,
+      EMPATHY_NOTIFY_MANAGER_CAP_DESKTOP_ENTRY,
+      g_variant_new_string ("empathy"));
+
+  return notification;
+}
